;(load "/LIBS/SCHEME/scheme.scm")
(load "/LIBS/SCHEME/unittest.scm")

(define (a b) b)
(define (b) 4)
(define (d b c) (+ b c))

(define (x y . z) (if (null? z)
                      y
                      (+ y (car z))
                      ))
(assert-check-true "define/lambda"
                   (= (a 3) 3)
                   (= (b) 4)
                   (= 107 (d 100 7))
                   (= (x 3) 3)
                   (= (x 3 4) 7)
                   (= (x 3 4 5) 7)
)
(assert-check-true "if"
                   (if #t #t #f))

(assert-check-false "if should be #f"
                    (if #t #f #t)
                    (if #f #t))

(define test-name "set!")
(define x 3)
(if #t (set! x 4) (set! x 5))
(assert-true test-name (= 4 x))
(define x 3)
(if #f (set! x 4) (set! x 5))

(define xx 0)
(assert-check-true test-name
             (= ((lambda (y) (set! y 3) y) 1) 3)
             (= 5 x)
             (= 4 ((lambda (y) (set! y (+ y 2)) y) 2))      ; y has one scope.
             (= 4 ((lambda (xx) (set! xx (+ xx 2)) xx) 2)))  ; x has two scopes.

(assert-check-true "and"
                   (and 3)
                   (and #t #t)
                   (and #t #t #t)
                   (and 1 3 2)
                   (= 2 (and 1 3 2))
                   (= 4 (and 1 3 2 4)))

(assert-check-false "and should be #f"
                    (and #f)
                    (and #t #f)
                    (and #t #f #t)
                    (and 1 #f 2)
                    (= 2 (and 1 #f 2))
                    (= 4 (and 1 #f 2 4)))

(assert-check-true "or"
                   (or 3)
                   (or #t)
                   (or #t #t)
                   (or #t #t #t)
                   (or 1 #f 2)
                   (or 1 3 2)
                   (= 2 (or 2 3 1))
                   (= 4 (or 4 3 2 4))
                   (= 2 (or #f 2 #f))
                   (= 4 (or #f #f 4)))

(assert-check-false "or should be #f"
                    (or #f)
                    (or #f #f)
                    (or #f #f #f))

(define x 0)
(assert-check-true "begin"
                   (= 4 (begin (+ 3 1)))
                   (= 4 (begin (set! x 1) 4))
                   (= x 1))

(assert-check-true "lambda"
                   (= 3 ((lambda (x) (+ x 1)) 2))
                   (= 3 ((lambda (x y) (+ x 1 y)) 1 1)))
(define x 0)
((lambda (x) (set! x 3)) 2)
;; todo
;; (assert-true test-name (= 0 x))

(let ((a 3) (b 2))
  (assert-true "let" (= 5 (+ a b))))
(let ((a 3))
  (let ((b 2))
    (assert-true "let" (= 5 (+ a b)))))

(define test-name "let*")
(let* ((a 3) (b 2))
  (assert-true "let*" (= 5 (+ a b))))
(let* ((a 3))
  (let* ((b 2))
    (assert-true "let*" (= 5 (+ a b)))))

(define test-name "cond")
(cond (#t (assert-true test-name #t))
      (else (assert-fail test-name "not reached")))
(cond (#f (assert-fail test-name "not reached"))
      (else (assert-true test-name #t)))
(cond (#f (assert-fail test-name "not reached"))
      (#t (assert-true test-name #t))
      (else (assert-fail test-name "not reached")))
(cond (#f (assert-fail test-name "not reached"))
      ((+ 1 2) (assert-true test-name #t))
      (else (assert-fail test-name "not reached")))
(cond (#f (assert-fail test-name "not reached"))
      (#t (assert-true test-name #t))
      (#f (assert-fail test-name "not reached")))

(assert-check-true "cond"
                   (= 3 (cond (#t 3)))
                   (= 4 (cond (#f 3) (#t 4) (#f 2))))

(define fact
  (lambda (n)
    (if (= n 0)
        1
        (* n (fact (- n 1))))))
(assert-true "fact" (= 120 (fact 5)))



(define test-string1 "hige")
(define test-string2 test-string1)
(define test-pair1 (cons 1 2))
(define test-pair2 test-pair1)
(assert-check-true "eqv? - should be #t"
                   (eqv? 'a 'a)
                   (eqv? #\a #\a)
                   (eqv? #t #t)
                   (eqv? #f #f)
                   (eqv? 3 3)
                   (eqv? (quote a) (quote a))
                   (eqv? (quote ()) (quote ()))
                   (eqv? test-string1 test-string2)
                   (eqv? test-pair1 test-pair2))

(define p (lambda (x) (+ x 1)))
(assert-check-false "eqv? - should be #f different type"
                    (eqv? 'a 'b)
                    (eqv? #\a #\b)
                    (eqv? #t 2)
                    (eqv? #t "false")
                    (eqv? #t (quote (a b)))
                    (eqv? #t (cons 1 2))
                    (eqv? 2 "number")
                    (eqv? 2 (quote (a b)))
                    (eqv? 2 (cons 1 2))
                    (eqv? (quote (x y)) (cons 1 2))
                    (eqv? p p))

(assert-check-false "eqv? - should be #f"
                    (eqv? #t #f)
                    (eqv? #f #t)
                    (eqv? (quote ()) 3)
                    (eqv? (quote a) (quote b))
                    (eqv? "string" "string")
                    (eqv? (cons 1 2) (cons 1 2))
                    (eqv? 2 3)
                    (eqv? (lambda () 1) (lambda () 2)))

(assert-check-false "eqv? - R5RS doesn't say anything about these. So same as Gauche."
                    (eqv? "" "")
                    (eqv? (lambda (x) x)
                          (lambda (x) x))
                    (eqv? (lambda (x) x)
                          (lambda (y) y)))

(assert-check-true "quote"
                   (eqv? (quote a) (quote a))
                   (eqv? (car (quote (x y z))) (quote x)))

(assert-check-true "quote '"
                   (eqv? (caar (cons (cons 3 4) (cons 5 6))) 3)
                   (eqv? 'a 'a)
                   (eqv? (car '(x y z)) 'x))
(assert-check-true "quote self evaluate"
                   '#t
                   (= '12345 12345)
                   (string=? '"abc" "abc"))

(assert-check-true "quote (a . b)"
                   (car '(#t . #f))
                   (cdr '(#f . #t))
                   (not (cdr '(#t . #f)))
)

(assert-check-true "should be #t"
                   0
                   1
                   '()
                   null
                   'a
                   #t
                   (cons 1 2)
                   (lambda (x) x))

(assert-check-false "should be #f" #f)

(assert-check-true "eq?"
                   (eq? 'a 'a)
                   (eq? '() '())
                   (eq? car car)
                   (let ((x '(a)))
                     (eq? x x))
                   (let ((x '#()))
                     (eq? x x))
                   (let ((p (lambda (x) x)))
                     (eq? p p)))

(assert-check-false "eq? should be #f"
                    (eq? (list 'a) (list 'a)))

(define xyz 1234)
(assert-true "quote"
             (= 1234 (eval (quote xyz) (scheme-report-environment 5))))

(eval (quote (define hoge+ (lambda (x) (+ 4 x)))) (scheme-report-environment 5))
(assert-true "eval" (= 4 (hoge+ 0)))
;; (eval (quote (define-syntax pqr
;;                 (syntax-rules ()
;;                  ((_ a) (+ a 3))))) (scheme-report-environment 5))
;; (assert-true "eval" (= 7 (pqr 4)))

(assert-check-true "apply"
             (= -1 (apply - (list 2 3)))
             (= -1 (apply (lambda (a b) (- a b)) (list 2 3))))

(assert-check-true "procedure?"
                   (procedure? +)
                   (procedure? (lambda () 3)))

(assert-check-false "procedure?"
                   (procedure? 1)
                   (procedure? (quote 3))
                   (procedure? "string")
                   (procedure? (cons 1 2)))

(assert-check-true "boolean?"
                   (boolean? #t)
                   (boolean? #f))

(assert-check-false "boolean?"
                   (boolean? 1)
                   (boolean? "hige"))

((lambda (x y . z)
   (assert-check-true "lambda"
                      (= x 3)
                      (= y 4)
                      (null? z)))
   3 4)

((lambda (x y . z)
   (assert-check-true "lambda"
                      (= x 3)
                      (= y 4)
                      (= (car z) 5)))
   3 4 5)

((lambda (x y . z)
   (assert-check-true "lambda"
                      (= x 3)
                      (= y 4)
                      (= (car z) 5)
                      (= (cadr z) 6)))
   3 4 5 6)

(let ((count 0))
  (let func ((l (list 1 2 3)))
    (if (null? l)
        0
        (begin
          (set! count (+ count (car l)))
          (func (cdr l)))))
(assert-check-true "named let" (= 6 count)))

(assert-check-true "case"
                   (case 3
                     ((5) #f)
                     ((4) #f)
                     (else #t))
                   (case 3
                     ((3) #t)
                     ((4) #f)
                     (else #f))
                   (case 3
                     ((5) #f)
                     ((3) #t)
                     (else #f))
                   (case (* 2 3)
                     ((2 3 5 7) #f)
                     ((1 4 6 8 9) #t))
)

(define do-index 0)
(assert-check-true "do"
                   (= 5
                      (do ((i 0 (+ i 1)))
                          ((= i 5) i)
                        (set! do-index (+ do-index 2))))
                   (= do-index 10))

(define do-index 0)
(assert-check-true "do"
                   (= 11
                      (do ((i 0 (+ i 1))
                           (j 1 (+ j 1)))
                          ((= i 5) (+ i j))
                        (set! do-index (+ do-index 2))))
                   (= do-index 10))

;; (define do-index 0)
;; (assert-check-true "do"
;;                    (do ((i 0 (+ i 1))
;;                            (j 1))
;;                           ((= i 5))
;;                         (set! do-index (+ do-index 2)))
;;                    (= do-index 10))

;; (define do-index 0)
;; (assert-check-true "do"
;;                    (do ((i 0 (+ i 1)))
;;                           ((= i 5))
;;                         (set! do-index (+ do-index 2)))
;;                    (= do-index 10))

;; (define-macro b
;;   (lambda (n)
;;   (list '+ 1 n)))

;; (define-macro foo
;;   (lambda ()
;;   '(+ 1 2)))

;; (assert-check-true "define-macro"
;;                    (= (bar 3) 4)
;;                    (= (foo) 3))

(load "/SERVERS/TEST/dir.scm")
(load "/SERVERS/TEST/cont.scm")
(load "/SERVERS/TEST/regexp.scm")
(load "/SERVERS/TEST/string.scm")
(load "/SERVERS/TEST/pair.scm")
(load "/SERVERS/TEST/util.scm")
(load "/SERVERS/TEST/port.scm")
(load "/SERVERS/TEST/char.scm")
(load "/SERVERS/TEST/vector.scm")
(load "/SERVERS/TEST/symbol.scm")
(load "/SERVERS/TEST/number.scm")
(load "/SERVERS/TEST/closure.scm")
(load "/SERVERS/TEST/shell.scm")

;; report
(total-report)
